import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:get/get.dart';
import 'package:uuid/uuid.dart';

import 'status_controller.dart';

class BaseController extends GetxController {
  StatusController _status = Get.find();

  List<StreamSubscription> _streams = [];
  Set<Uuid> _tasks = {};

  void autoDispose(StreamSubscription stream) => _streams.add(stream);

  @override
  void onClose() {
    _streams.forEach((element) => element.cancel());
    _streams.clear();

    _tasks.forEach((element) => _status.complete(element));
    _tasks.clear();

    super.onClose();
  }

  bool get isProcessing => _status.processes.isNotEmpty;

  Future<T?> enqueue<T>(
    Future<T> Function() task, {
    bool ignoreIfLoading = false,
  }) async {
    if (ignoreIfLoading && isProcessing) return null;
    final job = Uuid();
    try {
      _status.add(job);
      _tasks.add(job);
      return await task();
    } catch (error, stackTrace) {
      _status.catchError(error);
      if (kDebugMode) {
        print(stackTrace);
      }
      return null;
    } finally {
      _status.complete(job);
    }
  }

  void catchError(dynamic source) {
    _status.catchError(source);
  }

  Future addDummy({Duration after = const Duration(milliseconds: 500)}) async {
    await enqueue(() => Future.delayed(after));
  }
}
