import 'dart:async';

import 'package:flutter/scheduler.dart';
import 'package:get/get.dart';
import 'package:uuid/uuid.dart';

class StatusController extends GetxController {
  static Duration defaultTaskTimeout = Duration(seconds: 5);

  RxList<DartError> _exceptions = <DartError>[].obs;

  RxSet<Uuid> processes = <Uuid>{}.obs;
  RxBool isProcessing = false.obs;

  List<StreamSubscription> _streams = [];

  void autoDispose(StreamSubscription stream) => _streams.add(stream);

  @override
  void onInit() {
    super.onInit();
    processes.listen(
          (data) {
        SchedulerBinding.instance?.addPostFrameCallback(
              (timeStamp) {
            if (isProcessing.value == data.isNotEmpty) return;
            isProcessing.toggle();
          },
        );
      },
    );
  }

  @override
  void onClose() {
    _streams.forEach((element) => element.cancel());
    _streams.clear();
    super.onClose();
  }

  void add(Uuid uuid) {
    processes.add(uuid);
    Future.delayed(defaultTaskTimeout).then((value) => complete(uuid));
  }

  void complete(Uuid uuid) {
    processes.remove(uuid);
  }

  DartError? currentError() => _exceptions.isEmpty ? null : _exceptions.first;

  void catchError(dynamic error) {
    _exceptions.add(DartError(error));
  }

  void consumeError(DartError error) {
    _exceptions.removeWhere((element) => element.uuid == error.uuid);
  }
}

class DartError {
  dynamic error;
  String uuid = Uuid().v4();

  DartError(this.error);

  @override
  bool operator ==(Object other) =>
      (other is DartError) ? uuid == other.uuid : false;

  @override
  String toString() => error.toString();
}
